@@ -25,7 +25,11 @@
 #include <windows.h>
 #include "dialogs.h"
 #include "common.h"
-#include "engine.h"
+
+#include <string>
+
+#include <gpgme++/context.h>
+#include <gpgme++/data.h>
 
 /* To avoid writing a dialog template for each language we use gettext
    for the labels and hope that there is enough space in the dialog to
@@ -36,9 +40,6 @@ set_labels (HWND dlg)
   static struct { int itemid; const char *label; } labels[] = {
     { IDC_G_GENERAL,        N_("General")},
     { IDC_ENABLE_SMIME,     N_("Enable the S/MIME support")},
-#ifndef MIME_SEND
-    { IDC_MIME_UI,          N_("Enable simplified interface (experimental)")},
-#endif
 
     { IDC_G_SEND,           N_("Message sending")},
     { IDC_ENCRYPT_DEFAULT,  N_("&Encrypt new messages by default")},
@@ -61,20 +62,61 @@ set_labels (HWND dlg)
     SetDlgItemText (dlg, labels[i].itemid, _(labels[i].label));
 }
 
-
-/** Enable or disable options that are only relvant for mime ui */
 static void
-enable_disable_opts (HWND hDlg)
+launch_kleo_config (HWND hDlg)
 {
-  BOOL enable = opt.mime_ui ? TRUE : FALSE;
-  EnableWindow (GetDlgItem (hDlg, IDC_ENCRYPT_DEFAULT), enable);
-  EnableWindow (GetDlgItem (hDlg, IDC_SIGN_DEFAULT), enable);
   char *uiserver = get_uiserver_name ();
-  if (!uiserver && !opt.enable_smime)
+  bool showError = false;
+  if (uiserver)
     {
-      EnableWindow (GetDlgItem (hDlg, IDC_AUTORRESOLVE), FALSE);
+      std::string path (uiserver);
+      xfree (uiserver);
+      if (path.find("kleopatra.exe") != std::string::npos)
+        {
+        size_t dpos;
+        if ((dpos = path.find(" --daemon")) != std::string::npos)
+            {
+              path.erase(dpos, strlen(" --daemon"));
+            }
+          auto ctx = GpgME::Context::createForEngine(GpgME::SpawnEngine);
+          if (!ctx)
+            {
+              log_error ("%s:%s: No spawn engine.",
+                         SRCNAME, __func__);
+            }
+            std::string parentWid = std::to_string ((int) (intptr_t) hDlg);
+            const char *argv[] = {path.c_str(),
+                                  "--config",
+                                  "--parent-windowid",
+                                  parentWid.c_str(),
+                                  NULL };
+            log_debug ("%s:%s: Starting %s %s %s",
+                       SRCNAME, __func__, path.c_str(), argv[1], argv[2]);
+            GpgME::Data d(GpgME::Data::null);
+            ctx->spawnAsync(path.c_str(), argv, d, d,
+                            d, (GpgME::Context::SpawnFlags) (
+                                GpgME::Context::SpawnAllowSetFg |
+                                GpgME::Context::SpawnShowWindow));
+        }
+      else
+        {
+          showError = true;
+        }
     }
-  xfree (uiserver);
+  else
+    {
+      showError = true;
+    }
+
+  if (showError)
+    {
+      MessageBox (NULL,
+                  _("Could not find Kleopatra.\n"
+                  "Please reinstall Gpg4win with the Kleopatra component enabled."),
+                  _("GpgOL"),
+                  MB_ICONINFORMATION|MB_OK);
+    }
+
 }
 
 static INT_PTR CALLBACK
@@ -91,17 +133,12 @@ options_window_proc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
                               !!opt.encrypt_default, 0L);
           SendDlgItemMessage (hDlg, IDC_SIGN_DEFAULT, BM_SETCHECK,
                               !!opt.sign_default, 0L);
-#ifndef MIME_SEND
-          SendDlgItemMessage (hDlg, IDC_MIME_UI, BM_SETCHECK,
-                              !!opt.mime_ui, 0L);
-#endif
           SendDlgItemMessage (hDlg, IDC_INLINE_PGP, BM_SETCHECK,
                               !!opt.inline_pgp, 0L);
           SendDlgItemMessage (hDlg, IDC_REPLYCRYPT, BM_SETCHECK,
                               !!opt.reply_crypt, 0L);
           SendDlgItemMessage (hDlg, IDC_AUTORRESOLVE, BM_SETCHECK,
                               !!opt.autoresolve, 0L);
-          enable_disable_opts (hDlg);
           set_labels (hDlg);
           ShowWindow (GetDlgItem (hDlg, IDC_GPG_OPTIONS),
                       opt.enable_debug ? SW_SHOW : SW_HIDE);
@@ -123,18 +160,6 @@ options_window_proc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
                   (hDlg, IDC_ENCRYPT_DEFAULT, BM_GETCHECK, 0, 0L);
                 opt.sign_default = !!SendDlgItemMessage
                   (hDlg, IDC_SIGN_DEFAULT, BM_GETCHECK, 0, 0L);
-#ifndef MIME_SEND
-                int mime_ui_old = opt.mime_ui;
-                opt.mime_ui = !!SendDlgItemMessage
-                  (hDlg, IDC_MIME_UI, BM_GETCHECK, 0, 0L);
-                if (opt.mime_ui != mime_ui_old)
-                  {
-                    MessageBox (NULL,
-                                _("Changing the interface requires a restart of Outlook."),
-                                _("GpgOL"),
-                                MB_ICONINFORMATION|MB_OK);
-                  }
-#endif
                 opt.inline_pgp = !!SendDlgItemMessage
                   (hDlg, IDC_INLINE_PGP, BM_GETCHECK, 0, 0L);
 
@@ -149,7 +174,7 @@ options_window_proc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
                 break;
               }
             case IDC_GPG_CONF:
-              engine_start_confdialog (hDlg);
+              launch_kleo_config (hDlg);
               break;
             case IDC_GPG_OPTIONS:
               config_dialog_box (hDlg);
