@@ -38,6 +38,8 @@
 
 #include "common.h"
 
+#include "dialogs.h"
+
 HINSTANCE glob_hinst = NULL;
 
 void
@@ -242,6 +244,24 @@ get_save_filename (HWND root, const char *srcname)
   return NULL;
 }
 
+void
+bring_to_front (HWND wid)
+{
+  if (wid)
+    {
+      if (!SetForegroundWindow (wid))
+        {
+          log_debug ("%s:%s: SetForegroundWindow failed", SRCNAME, __func__);
+          /* Yet another fallback which will not work on some
+           * versions and is not recommended by msdn */
+          if (!ShowWindow (wid, SW_SHOWNORMAL))
+            {
+              log_debug ("%s:%s: ShowWindow failed.", SRCNAME, __func__);
+            }
+        }
+    }
+  log_debug ("%s:%s: done", SRCNAME, __func__);
+}
 
 void
 fatal_error (const char *format, ...)
@@ -1011,3 +1031,177 @@ is_elevated()
 
   return ret;
 }
+
+int
+gpgol_message_box (HWND parent, const char *utf8_text,
+                   const char *utf8_caption, UINT type)
+{
+  wchar_t *w_text = utf8_to_wchar (utf8_text);
+  wchar_t *w_caption = utf8_to_wchar (utf8_caption);
+  int ret = 0;
+
+  MSGBOXPARAMSW mbp;
+  mbp.cbSize = sizeof (MSGBOXPARAMS);
+  mbp.hwndOwner = parent;
+  mbp.hInstance = glob_hinst;
+  mbp.lpszText = w_text;
+  mbp.lpszCaption = w_caption;
+  mbp.dwStyle = type | MB_USERICON;
+  mbp.dwLanguageId = MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT);
+  mbp.lpfnMsgBoxCallback = NULL;
+  mbp.dwContextHelpId = 0;
+  mbp.lpszIcon = (LPCWSTR) MAKEINTRESOURCE (IDI_GPGOL_LOCK_ICON);
+
+  ret = MessageBoxIndirectW (&mbp);
+
+  xfree (w_text);
+  xfree (w_caption);
+  return ret;
+}
+
+void
+gpgol_bug (HWND parent, int code)
+{
+  const char *bugmsg = utf8_gettext ("Operation failed.\n\n"
+                "This is usually caused by a bug in GpgOL or an error in your setup.\n"
+                "Please see https://www.gpg4win.org/reporting-bugs.html "
+                "or ask your Administrator for support.");
+  char *with_code;
+  gpgrt_asprintf (&with_code, "%s\nCode: %i", bugmsg, code);
+  gpgol_message_box (parent,
+                     with_code,
+                     _("GpgOL Error"), MB_OK);
+  xfree (with_code);
+  return;
+}
+
+static char*
+expand_path (const char *path)
+{
+  DWORD len;
+  char *p;
+
+  len = ExpandEnvironmentStrings (path, NULL, 0);
+  if (!len)
+    {
+      return NULL;
+    }
+  len += 1;
+  p = xcalloc (1, len+1);
+  if (!p)
+    {
+      return NULL;
+    }
+  len = ExpandEnvironmentStrings (path, p, len);
+  if (!len)
+    {
+      xfree (p);
+      return NULL;
+    }
+  return p;
+}
+
+static int
+load_config_value (HKEY hk, const char *path, const char *key, char **val)
+{
+  HKEY h;
+  DWORD size=0, type;
+  int ec;
+
+  *val = NULL;
+  if (hk == NULL)
+    {
+      hk = HKEY_CURRENT_USER;
+    }
+  ec = RegOpenKeyEx (hk, path, 0, KEY_READ, &h);
+  if (ec != ERROR_SUCCESS)
+    {
+      return -1;
+    }
+
+  ec = RegQueryValueEx(h, key, NULL, &type, NULL, &size);
+  if (ec != ERROR_SUCCESS)
+    {
+      RegCloseKey (h);
+      return -1;
+    }
+  if (type == REG_EXPAND_SZ)
+    {
+      char tmp[256];
+      RegQueryValueEx (h, key, NULL, NULL, (BYTE*)tmp, &size);
+      *val = expand_path (tmp);
+    }
+  else
+    {
+      *val = xcalloc(1, size+1);
+      ec = RegQueryValueEx (h, key, NULL, &type, (BYTE*)*val, &size);
+      if (ec != ERROR_SUCCESS)
+        {
+          xfree (*val);
+          *val = NULL;
+          RegCloseKey (h);
+          return -1;
+        }
+    }
+  RegCloseKey (h);
+  return 0;
+}
+
+static int
+store_config_value (HKEY hk, const char *path, const char *key, const char *val)
+{
+  HKEY h;
+  int type;
+  int ec;
+
+  if (hk == NULL)
+    {
+      hk = HKEY_CURRENT_USER;
+    }
+  ec = RegCreateKeyEx (hk, path, 0, NULL, REG_OPTION_NON_VOLATILE,
+                       KEY_ALL_ACCESS, NULL, &h, NULL);
+  if (ec != ERROR_SUCCESS)
+    {
+      log_debug_w32 (ec, "creating/opening registry key `%s' failed", path);
+      return -1;
+    }
+  type = strchr (val, '%')? REG_EXPAND_SZ : REG_SZ;
+  ec = RegSetValueEx (h, key, 0, type, (const BYTE*)val, strlen (val));
+  if (ec != ERROR_SUCCESS)
+    {
+      log_debug_w32 (ec, "saving registry key `%s'->`%s' failed", path, key);
+      RegCloseKey(h);
+      return -1;
+    }
+  RegCloseKey(h);
+  return 0;
+}
+
+/* Store a key in the registry with the key given by @key and the
+   value @value. */
+int
+store_extension_value (const char *key, const char *val)
+{
+  return store_config_value (HKEY_CURRENT_USER, GPGOL_REGPATH, key, val);
+}
+
+/* Load a key from the registry with the key given by @key. The value is
+   returned in @val and needs to freed by the caller. */
+int
+load_extension_value (const char *key, char **val)
+{
+  return load_config_value (HKEY_CURRENT_USER, GPGOL_REGPATH, key, val);
+}
+
+int
+store_extension_subkey_value (const char *subkey,
+                              const char *key,
+                              const char *val)
+{
+  int ret;
+  char *path;
+  gpgrt_asprintf (&path, "%s\\%s", GPGOL_REGPATH, subkey);
+  ret = store_config_value (HKEY_CURRENT_USER, path, key, val);
+  xfree (path);
+  return ret;
+}
