@@ -28,13 +28,11 @@
 #include "attachment.h"
 #include "mapihelp.h"
 #include "mimemaker.h"
-#include "message.h"
 #include "revert.h"
 #include "gpgoladdin.h"
 #include "mymapitags.h"
 #include "parsecontroller.h"
 #include "cryptcontroller.h"
-#include "gpgolstr.h"
 #include "windowmessages.h"
 #include "mlang-charset.h"
 #include "wks-helper.h"
@@ -94,6 +92,7 @@ Mail::Mail (LPDISPATCH mailitem) :
     m_window(nullptr),
     m_async_crypt_disabled(false),
     m_is_forwarded_crypto_mail(false),
+    m_is_reply_crypto_mail(false),
     m_is_send_again(false),
     m_disable_att_remove_warning(false)
 {
@@ -1084,11 +1083,77 @@ Mail::update_body()
   // No need to carry body anymore
   m_orig_body = std::string();
   auto html = m_parser->get_html_body ();
+  auto body = m_parser->get_body ();
   /** Outlook does not show newlines if \r\r\n is a newline. We replace
     these as apparently some other buggy MUA sends this. */
   find_and_replace (html, "\r\r\n", "\r\n");
   if (opt.prefer_html && !html.empty())
     {
+      if (!m_block_html)
+        {
+          const auto charset = m_parser->get_html_charset();
+
+          int codepage = 0;
+          if (charset.empty())
+            {
+              codepage = get_oom_int (m_mailitem, "InternetCodepage");
+              log_debug ("%s:%s: Did not find html charset."
+                         " Using internet Codepage %i.",
+                         SRCNAME, __func__, codepage);
+            }
+
+          char *converted = ansi_charset_to_utf8 (charset.c_str(), html.c_str(),
+                                                  html.size(), codepage);
+          int ret = put_oom_string (m_mailitem, "HTMLBody", converted ?
+                                                            converted : "");
+          xfree (converted);
+          if (ret)
+            {
+              log_error ("%s:%s: Failed to modify html body of item.",
+                         SRCNAME, __func__);
+            }
+
+          return;
+        }
+      else if (!body.empty())
+        {
+          /* We had a multipart/alternative mail but html should be
+             blocked. So we prefer the text/plain part and warn
+             once about this so that we hopefully don't get too
+             many bugreports about this. */
+          if (!opt.smime_html_warn_shown)
+            {
+              std::string caption = _("GpgOL") + std::string (": ") +
+                std::string (_("HTML display disabled."));
+              std::string buf = _("HTML content in unsigned S/MIME mails "
+                                  "is insecure.");
+              buf += "\n";
+              buf += _("GpgOL will only show such mails as text.");
+
+              buf += "\n\n";
+              buf += _("This message is shown only once.");
+
+              gpgol_message_box (get_window(), buf.c_str(), caption.c_str(),
+                                 MB_OK);
+              opt.smime_html_warn_shown = true;
+              write_options ();
+            }
+        }
+    }
+
+  if (body.empty () && m_block_html && !html.empty())
+    {
+#if 0
+      Sadly the following code still offers to load external references
+      it might also be too dangerous if Outlook somehow autoloads the
+      references as soon as the Body is put into HTML
+
+
+      // Fallback to show HTML as plaintext if HTML display
+      // is blocked.
+      log_error ("%s:%s: No text body. Putting HTML into plaintext.",
+                 SRCNAME, __func__);
+
       char *converted = ansi_charset_to_utf8 (m_parser->get_html_charset().c_str(),
                                               html.c_str(), html.size());
       int ret = put_oom_string (m_mailitem, "HTMLBody", converted ? converted : "");
@@ -1097,13 +1162,67 @@ Mail::update_body()
         {
           log_error ("%s:%s: Failed to modify html body of item.",
                      SRCNAME, __func__);
+          body = html;
         }
-      return;
+      else
+        {
+          char *plainBody = get_oom_string (m_mailitem, "Body");
+
+          if (!plainBody)
+            {
+              log_error ("%s:%s: Failed to obtain converted plain body.",
+                         SRCNAME, __func__);
+              body = html;
+            }
+          else
+            {
+              ret = put_oom_string (m_mailitem, "HTMLBody", plainBody);
+              xfree (plainBody);
+              if (ret)
+                {
+                  log_error ("%s:%s: Failed to put plain into html body of item.",
+                             SRCNAME, __func__);
+                  body = html;
+                }
+              else
+                {
+                  return;
+                }
+            }
+        }
+#endif
+      body = html;
+      std::string caption = _("GpgOL") + std::string (": ") +
+        std::string (_("HTML display disabled."));
+      std::string buf = _("HTML content in unsigned S/MIME mails "
+                          "is insecure.");
+      buf += "\n";
+      buf += _("GpgOL will only show such mails as text.");
+
+      buf += "\n\n";
+      buf += _("Please ask the sender to sign the message or\n"
+               "to send it with a plain text alternative.");
+
+      gpgol_message_box (get_window(), buf.c_str(), caption.c_str(),
+                         MB_OK);
     }
-  auto body = m_parser->get_body ();
+
   find_and_replace (body, "\r\r\n", "\r\n");
-  char *converted = ansi_charset_to_utf8 (m_parser->get_body_charset().c_str(),
-                                          body.c_str(), body.size());
+
+  const auto plain_charset = m_parser->get_body_charset();
+
+  int codepage = 0;
+  if (plain_charset.empty())
+    {
+      codepage = get_oom_int (m_mailitem, "InternetCodepage");
+      log_debug ("%s:%s: Did not find body charset. "
+                 "Using internet Codepage %i.",
+                 SRCNAME, __func__, codepage);
+    }
+
+  char *converted = ansi_charset_to_utf8 (plain_charset.c_str(),
+                                          body.c_str(), body.size(),
+                                          codepage);
   int ret = put_oom_string (m_mailitem, "Body", converted ? converted : "");
   xfree (converted);
   if (ret)
@@ -1162,7 +1281,16 @@ Mail::parsing_done()
 
   TRACEPOINT;
   /* Set categories according to the result. */
-  update_categories();
+  update_categories ();
+
+  TRACEPOINT;
+  m_block_html = m_parser->shouldBlockHtml ();
+
+  if (m_block_html)
+    {
+      // Just to be careful.
+      set_block_status ();
+    }
 
   TRACEPOINT;
   /* Update the body */
@@ -2280,6 +2408,11 @@ Mail::get_crypto_details()
                m_verify_result.numSignatures() < 1)
         {
           message += _("There was an error verifying the signature.\n");
+          const auto err = m_sig.status ();
+          if (err)
+            {
+              message += err.asString () + std::string ("\n");
+            }
         }
       else if (m_sig.summary() & Signature::Summary::SigExpired)
         {
@@ -2577,6 +2710,9 @@ Mail::inline_body_to_body()
       return 0;
     }
 
+  /* For inline we always work with UTF-8 */
+  put_oom_int (m_mailitem, "InternetCodepage", 65001);
+
   int ret = put_oom_string (m_mailitem, "Body",
                             body.c_str ());
   return ret;
@@ -2840,6 +2976,51 @@ Mail::locate_all_crypto_recipients()
     }
 }
 
+int
+Mail::remove_all_attachments ()
+{
+  int ret = 0;
+  LPDISPATCH attachments = get_oom_object (m_mailitem, "Attachments");
+  if (!attachments)
+    {
+      TRACEPOINT;
+      return 0;
+    }
+  int count = get_oom_int (attachments, "Count");
+  LPDISPATCH to_delete[count];
+
+  /* Populate the array so that we don't get in an index mess */
+  for (int i = 1; i <= count; i++)
+    {
+      auto item_str = std::string("Item(") + std::to_string (i) + ")";
+      to_delete[i-1] = get_oom_object (attachments, item_str.c_str());
+    }
+  gpgol_release (attachments);
+
+  /* Now delete all attachments */
+  for (int i = 0; i < count; i++)
+    {
+      LPDISPATCH attachment = to_delete[i];
+
+      if (!attachment)
+        {
+          log_error ("%s:%s: No such attachment %i",
+                     SRCNAME, __func__, i);
+          ret = -1;
+        }
+
+      /* Delete the attachments that are marked to delete */
+      if (invoke_oom_method (attachment, "Delete", NULL))
+        {
+          log_error ("%s:%s: Deleting attachment %i",
+                     SRCNAME, __func__, i);
+          ret = -1;
+        }
+      gpgol_release (attachment);
+    }
+  return ret;
+}
+
 int
 Mail::remove_our_attachments ()
 {
@@ -2968,3 +3149,30 @@ Mail::get_verification_result_dump()
   ss << m_verify_result;
   return ss.str();
 }
+
+void
+Mail::set_block_status()
+{
+  SPropValue prop;
+
+  LPMESSAGE message = get_oom_base_message (m_mailitem);
+
+  prop.ulPropTag = PR_BLOCK_STATUS;
+  prop.Value.l = 1;
+  HRESULT hr = message->SetProps (1, &prop, NULL);
+
+  if (hr)
+    {
+      log_error ("%s:%s: can't set block value: hr=%#lx\n",
+                 SRCNAME, __func__, hr);
+    }
+
+  gpgol_release (message);
+  return;
+}
+
+void
+Mail::set_block_html(bool value)
+{
+  m_block_html = value;
+}
