@@ -77,7 +77,8 @@ ParseController::ParseController(LPSTREAM instream, msgtype_t type):
     m_inputprovider  (new MimeDataProvider(instream,
                           expect_no_headers(type))),
     m_outputprovider (new MimeDataProvider(expect_no_mime(type))),
-    m_type (type)
+    m_type (type),
+    m_block_html (false)
 {
   log_mime_parser ("%s:%s: Creating parser for stream: %p of type %i"
                    " expect no headers: %i expect no mime: %i",
@@ -90,7 +91,8 @@ ParseController::ParseController(FILE *instream, msgtype_t type):
     m_inputprovider  (new MimeDataProvider(instream,
                           expect_no_headers(type))),
     m_outputprovider (new MimeDataProvider(expect_no_mime(type))),
-    m_type (type)
+    m_type (type),
+    m_block_html (false)
 {
   log_mime_parser ("%s:%s: Creating parser for stream: %p of type %i",
                    SRCNAME, __func__, instream, type);
@@ -200,6 +202,11 @@ format_error(GpgME::DecryptionResult result, Protocol protocol)
         {
           msg += _("Failed to parse the mail.");
         }
+      else if (result.isLegacyCipherNoMDC())
+        {
+          msg += _("Data is not integrity protected. "
+                   "Decrypting it could be a security problem. (no MDC)");
+        }
       else
         {
           msg += result.error().asString();
@@ -226,6 +233,25 @@ ParseController::setSender(const std::string &sender)
   m_sender = sender;
 }
 
+static bool
+is_valid_chksum(const GpgME::Signature &sig)
+{
+  switch (sig.summary())
+    {
+      case GpgME::Signature::Valid:
+      case GpgME::Signature::Green:
+      case GpgME::Signature::KeyRevoked:
+      case GpgME::Signature::KeyExpired:
+      case GpgME::Signature::SigExpired:
+      case GpgME::Signature::CrlMissing:
+      case GpgME::Signature::CrlTooOld:
+      case GpgME::Signature::TofuConflict:
+        return true;
+      default:
+        return false;
+    }
+}
+
 void
 ParseController::parse()
 {
@@ -350,6 +376,65 @@ ParseController::parse()
       else
         {
           m_verify_result = ctx->verifyOpaqueSignature(input, output);
+
+          const auto sigs = m_verify_result.signatures();
+          bool allBad = sigs.size();
+          for (const auto s :sigs)
+            {
+              if (!(s.summary() & Signature::Red))
+                {
+                  allBad = false;
+                  break;
+                }
+            }
+
+          if (allBad)
+            {
+              log_debug ("%s:%s:%p inline verify error trying native to utf8.",
+                         SRCNAME, __func__, this);
+
+
+              /* The proper solution would be to take the encoding from
+                 the mail / headers. Then convert the wchar body to that
+                 encoding. Verify, and convert it after verifcation to
+                 UTF-8 which the rest of the code expects.
+
+                 Or native_body from native ACP to InternetCodepage, then
+                 verify and convert the output back to utf8 as the rest
+                 expects.
+
+                 But as this is clearsigned and we don't really want that.
+                 Meh.
+                 */
+              char *utf8 = native_to_utf8 (input.toString().c_str());
+              if (utf8)
+                {
+                  // Try again after conversion.
+                  delete ctx;
+                  ctx = Context::createForProtocol (protocol);
+                  ctx->setArmor (true);
+                  if (!m_sender.empty())
+                    {
+                      ctx->setSender(m_sender.c_str());
+                    }
+
+                  input = Data (utf8, strlen (utf8));
+                  xfree (utf8);
+
+                  // Use a fresh output
+                  auto provider = new MimeDataProvider (true);
+
+                  // Warning: The dtor of the Data object touches
+                  // the provider. So we have to delete it after
+                  // the assignment.
+                  output = Data (provider);
+                  delete m_outputprovider;
+                  m_outputprovider = provider;
+
+                  // Try again
+                  m_verify_result = ctx->verifyOpaqueSignature(input, output);
+                }
+            }
         }
     }
   delete ctx;
@@ -358,9 +443,14 @@ ParseController::parse()
              m_verify_result.error().code());
 
   TRACEPOINT;
-  /* Ensure that the Keys for the signatures are available */
+
+  bool has_valid_encrypted_checksum = false;
+  /* Ensure that the Keys for the signatures are available
+     and if it has a valid encrypted checksum. */
   for (const auto sig: m_verify_result.signatures())
     {
+      has_valid_encrypted_checksum = is_valid_chksum (sig);
+
       sig.key(true, true);
       if (sig.validity() == Signature::Validity::Full ||
           sig.validity() == Signature::Validity::Ultimate)
@@ -371,6 +461,14 @@ ParseController::parse()
         }
     }
 
+  if (protocol == Protocol::CMS && decrypt && !m_decrypt_result.error() &&
+      !has_valid_encrypted_checksum)
+    {
+      log_debug ("%s:%s:%p Encrypted S/MIME without checksum. Block HTML.",
+                 SRCNAME, __func__, this);
+      m_block_html = true;
+    }
+
   if (opt.enable_debug)
     {
        std::stringstream ss;
