@@ -52,6 +52,7 @@ typedef enum _gpgol_wmsg_type
   BRING_TO_FRONT, /* Bring the active Outlook window to the front. */
   INVALIDATE_LAST_MAIL,
   REVERT_MAIL,
+  CLEAR_REPLY_FORWARD,
 } gpgol_wmsg_type;
 
 typedef struct
@@ -59,6 +60,7 @@ typedef struct
   void *data; /* Pointer to arbitrary data depending on msg type */
   gpgol_wmsg_type wmsg_type; /* Type of the msg. */
   int err; /* Set to true on error */
+  int delay;
 } wm_ctx_t;
 
 /** Create and register the responder window.
@@ -77,9 +79,12 @@ int
 do_in_ui_thread (gpgol_wmsg_type type, void *data);
 
 /** Send a message to the UI thread but returns
-    immediately without waiting for the execution. */
+    immediately without waiting for the execution.
+
+    The delay is used in the detached thread to delay
+    the sending of the actual message. */
 void
-do_in_ui_thread_async (gpgol_wmsg_type type, void *data);
+do_in_ui_thread_async (gpgol_wmsg_type type, void *data, int delay = 0);
 
 /** Create our filter before outlook Window Messages. */
 HHOOK
